﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace TestCustomColumn
{
    public class ExpandColumn : TextColumn
    {
        protected override ColumnContentProviderBase GenerateContentProvider()
        {
            return new ExpandColumnContentProvider();
        }

        public event EventHandler<ExpandColumnClickEventArgs> Click;
        protected internal void OnClick(ExpandColumnClickEventArgs e)
        {
            if (Click != null)
            {
                this.Click(this, e);
            }
        }
    }

    public class ExpandColumnClickEventArgs : EventArgs
    {
        int _index;
        object _data;

        internal ExpandColumnClickEventArgs(int index, object data)
        {
            _index = index;
            _data = data;
        }

        public int Index { get { return _index; } }
        public object Data { get { return _data; } }
    }

    public class ExpandColumnContentProvider : TextColumnContentProvider
    {
        TextBlock expandBlock;
        TextBlock dataBlock;
        Cell _cell;
        ExpandColumn _column;

        public ExpandColumnContentProvider()
            : base()
        {
            expandBlock = GetCleanExpandBlock();
            dataBlock = new TextBlock();
            expandBlock.MouseLeftButtonDown += new System.Windows.Input.MouseButtonEventHandler(expandBlock_MouseLeftButtonDown);
        }

        private TextBlock GetCleanExpandBlock()
        {
            return new TextBlock()
            {
                VerticalAlignment = VerticalAlignment.Center,
                MinWidth = 10,
                FontWeight = FontWeights.Bold,
                FontSize = 10,
                Visibility = Visibility.Collapsed,
                Text = "+"
            };
        }

        public override void ResetContent()
        {
            expandBlock = GetCleanExpandBlock();
            _cell = null;
            _column = null;
            dataBlock = new TextBlock();
            base.ResetContent();
        }

        void expandBlock_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var r = (Row)_cell.Row;
            r.IsExpanded = !r.IsExpanded;
            ExpandColumnClickEventArgs _clickeventargs =
                new ExpandColumnClickEventArgs(_cell.Row.Index, _cell.Row.Data);
            _column.OnClick(_clickeventargs);
        }

        public override FrameworkElement ResolveDisplayElement(Cell cell, Binding cellBinding)
        {
            _column = (ExpandColumn)cell.Column;

            if (((Row)cell.Row).HasChildren && cell.Value != null)
                expandBlock.Visibility = Visibility.Visible;
            else
                expandBlock.Visibility = Visibility.Collapsed;
            if (((Row)cell.Row).IsExpanded)
                expandBlock.Text = "-";
            else
                expandBlock.Text = "+";

            dataBlock = base.ResolveDisplayElement(cell, cellBinding) as TextBlock;

            var p = new StackPanel()
            {
                Orientation = Orientation.Horizontal
            };
            p.Children.Add(expandBlock);
            p.Children.Add(dataBlock);

            return p;
        }

        public override void AdjustDisplayElement(Infragistics.Controls.Grids.Cell cell)
        {
            base.AdjustDisplayElement(cell);
            this._cell = cell;
            if (((Row)cell.Row).IsExpanded)
                expandBlock.Text = "-";
            else
                expandBlock.Text = "+";
        }
    }
}
